#include <Array.au3>

#cs
	Approximation von einem Dezimalbruch N
	durch Addition von 1/(2^x), x in Natürlichen Zahlen

	Hintergrund:
		Im x86-Befehlssatz habe ich zum Potenzieren keine wirklichen Funktionen gefunden bis auf F2XM1.
		F2XM1 berechnet (2^x)-1 und da ich x^(1/3) rechnen musste bietet sich da die
		Umrechnungsformel x^y = 2^(y * log2(x)) an, allerdings darf beim F2XM1 Befehl der Exponent x
		nur zwischen -1.0 und +1.0 liegen. Damit konnte ich leider nicht den gesamten Wertebereich von
		0.0 bis 1.0 hoch 1/3 abdecken -> 2^(1/3 * log2(0.0 bis 1.0)) da bei einem x von 0.125 (wird zu -1)
		und 1.0 (wird zu 0) abdecken.

		Das reichte mir leider immer noch nicht, da ich gerne eine Abdeckung von 0.0 (wird zu -unendlich) bis 1.0 wollte.

		Dann kam mir die Idee, einfach den Exponenten mittels der Wurzelfunktion zu approximieren.
		Es gibt im x86-Befehlssatz die Funktion FSQRT und diese berechnet die Quadratwurzel von ST(0) (oberster Wert auf dem FPU-Stack).

		Die Potenzgesetze erlauben es einem den Exponent zu addieren, wenn man Potenzen gleicher Basen multiplziert.
		Beispiel: 2^(1/2) * 2^(1/4) = 2^(1/2 + 1/4) = 2^(3/4)

		Jetzt kann man sich immer näher an den gewünschten Wert mittels Quadratwurzeln rantasten:
		Beispiel: 1/6 = 0.166666666

		Approximationsanfang = 0
		1/2^0 = 1/1 = 1				1 + Approximation <= 1/6 ? 				Nein!
		1/2^1 = 1/2 = 0.5			0.5 + Approximation <= 1/6 ? 			Nein!
		1/2^2 = 1/4 = 0.25			0.25 + Approximation <= 1/6 ? 			Nein!
		1/2^3 = 1/8 = 0.125			0.125 + Approximation <= 1/6 ?			Ja! 0.125 + 0 = 0.0125 				=> Neue Approximation = 0.125
		1/2^4 = 1/16 = 0.0625		0.0625 + Approximation <= 1/6 ? 		Nein!
		1/2^5 = 1/32 = 0.03125		0.03125 + Approximation <= 1/6 ?		Ja! 0.03125 + 0.125 = 0.15625		=> Neue Approximation = 0.15625
		1/2^6 = 1/64 = 0.015625		0.015625 + Approximation <= 1/6 ?		Nein!
		1/2^7 = 1/128 = 0.0078125	0.0078125 + Approximation <= 1/6 ? 		Ja!	0.15625 + 0.0078125 = 0.1640625	=> Neue Approximation = 0.1640625

		Wenn wir nun 0.1640625 als Approximation nutzen wollen müssen wir lediglich folgende Rechnung durchführen:

		x^(1/8) * x^(1/32) * x^(1/128)

		Als ASM-Code ungefähr so zu implementieren:

		x -> ST0			| ST0 = x
		SQRT(ST0) -> ST0	| ST0 = x^(1/2)
		SQRT(ST0) -> ST0	| ST0 = (x^(1/2))^(1/2) = x^((1/2) * (1/2)) = x^(1/4)
		SQRT(ST0) -> ST0	| ST0 = x^(1/8)

		Wir haben nun einen Wert den wir für die Approximation brauchen also kopieren wir ST0 nach ST1
		FST(ST1)  			|	ST0 = ST0, ST1 = ST0

		SQRT(ST0) -> ST0	| ST0 = x^(1/16)
		SQRT(ST0) -> ST0	| ST0 = x^(1/32)

		Neuer benötigter Wert!
		FST(ST2)			| ST0 = x^(1/32), ST1 = x^(1/8)

		SQRT(ST0) -> ST0	| ST0 = x^(1/64)
		SQRT(ST0) -> ST0	| ST0 = x^(1/128)

		Neuer benötigter Wert steht in ST0.
		Wir haben die Approximatino nun ausgerechnet und müssen nur noch die Ergebnisse multiplizieren:

		FMULP				| ST0 = x^(1/128) * x^(1/32), ST1 = x^(1/8) FMULP popt nämlich das Register nach der Multiplikation
		FMULP				| ST0 = x^(1/128) * x^(1/32) * x^(1/8)

		Fertig gerechnet, das Ergebnis steht in ST0!

		Man kann das ganze natürlich in ASM so programmieren, dann man beliebig nah an die Approximation kommt (wie das AutoIt-Äquivalent unten)
		aber in meinem Code haben vier Faktoren als Genauigkeit ausgereicht also habe ich es hardgecodet und nicht dymanisch.
#ce

$fApproxTo = 1/6

$aApproximation = _NthRoot($fApproxTo)
_ArrayDisplay($aApproximation, "Approx. of (1/9)")

Func _NthRoot($iApproxTo, $iDepth = 32)
	Local $fApproximation = 0, $aApproximation[1]

	For $i = 0 To $iDepth - 1
		If $fApproximation + (1/2^$i) <= $iApproxTo Then
			$fApproximation += (1/2^$i)
			ReDim $aApproximation[UBound($aApproximation) + 1]
			$aApproximation[UBound($aApproximation) - 1] = "1/" & 2^$i
		EndIf
	Next

	$aApproximation[0] = $fApproximation
	Return $aApproximation
EndFunc